
#include "eel.h"
#include "util.h"

/* from search.e */
#ifdef UNIX		/* i-search redoes cmd unless \r or ENDER */
#define ENDER	'\r'		/* only \r: return replaces ESC */
#else
#define ENDER	ESC		/* allow both */
#endif

/* Stever's modified version of replace string, which preserves case:
 *
 * Upgraded for Epsilon 6.5   6/2/93
 * Upgraded for Epsilon 6.03  11/24/92
 * 
 * Replace 'old-string' with 'new-string':
 * 
 * If 'case-fold' is FALSE, do an exact replace.
 *
 * Otherwise, 'old-string's capitalization in the buffer signals how to
 * casify the replace:
 *            if 'new-string' has any uppercase characters, do exact replace.
 *            otherwise, replace:
 *              old-string     in the buffer with   new-string
 *              Old-string     in the buffer with   New-string
 *              OLD-STRING     in the buffer with   NEW-STRING
 *
 */

/* Taken from Epsilon 6.03 sources */
/************************************************************************
* "Epsilon" is a registered trademark licensed to Lugaru Software, Ltd. *
*	"EEL" and "Lugaru" are trademarks of Lugaru Software, Ltd.	*
*									*
*  Copyright (C) 1985, 1992 Lugaru Software Ltd.  All rights reserved.	*
*									*
* Limited permission is hereby granted to reproduce and modify this	*
* copyrighted material provided that the resulting code is used only in	*
* conjunction with Lugaru products and that this notice is retained in	*
* any such reproduction or modification.				*
************************************************************************/

/* Interprets flag INCR to mean "interactive, show results". */

/* Returns DSABORT if the user aborted, DSBAD if illegal search string, */
/* 1 if user said to exit early, or 0 if we got to the last match. */

/* Epsilon 6.03 version:

string_replace(str, with, flags)
char *str, *with;
{
	spot orig = alloc_spot(), prev = alloc_spot(), oldmark;
	int skip = 0, show = 0, ret, evershow = 0, found = 0, changed = 0;
        int capitalize = case_fold;     // SER
        char *capWith = malloc(strlen(with)+1); // SER
        char *upWith = malloc(strlen(with)+1); // SER

        if (case_fold) {                // SER
             char *s;                   // SER
             for (s = with; *s && !isupper(*s); *s++) ; // SER
             if (isupper(*s)) capitalize = FALSE; // SER
             }                          // SER

        if (capitalize) {               // SER
             strcpy (capWith, with);    // SER
             strcpy (upWith,  with);    // SER
             capitalize_string (capWith); // SER
             uppercase_string (upWith); // SER
             }
   
	for (;;) {
		skip = 0;
		*prev = point;
		ret = do_searching(flags, str);
		if (!ret || ret == DSBAD) {
			point = *orig;
			break;
		} else if (ret == DSABORT)
			break;
		found++;
ask:		if (show) {
			show_replace(str, with, (flags & ~INCR));
			evershow = 1;
			show = 0;
		}
		if (!(flags & QUERY)) goto do_replace;
		maybe_refresh();        // show the selection
		getkey();
		if (key == CTRL('^'))
			full_getkey("", CTRLIFY_KEY);
		user_abort = 0;
		if (key == '?' || key == 'h'
					|| root_keys[key] == (short) help) {
			help_on_command("query-replace");
			show = 1;
			goto ask;
		} else if (user_abort || key == abort_key) {
			point = *orig;
			ret = DSABORT;
			goto done;
		}
		switch(tolower(key)) {
			case '^': point = *prev;
			case '\b':
			case 'n': skip = 1;	// go to the next match
			case ' ':
			case 'y':		// do the replace
			case '!':
			case ',':
			case '.': break;
			case CTRL('R'):
				say("");
				oldmark = alloc_spot();
				*oldmark = mark;	// save mark
				mark = point - strlen(skip? with: str);
					// set temporary mark before text
				recursive_edit();
				mark = *oldmark;	// restore mark
				free_spot(oldmark);
				show = 1;
				continue;
			default:	if (set_flag_modes(&flags)) {
						show = 1;
						goto ask;
					}
					if (!IS_ENTER_KEY(key))
						ungot_key = key;
			case ESC:	ret = 1; goto done;
		}
do_replace:	if (!skip) {
			if (flags & REGEX)
				fromorig(with);
			else {
                             if (!capitalize) { // SER
                                  delete(point - strlen(str), point);
                                  stuff(with);
                                  }
                             else capital_stuff (strlen(str), with, capWith,
                                                 upWith); // SER
			}
			changed++;
		}
		if (flags & QUERY)
			switch (key) {
			case ',': case '^':	skip = 1; goto ask;
			case '.':		ret = 1; goto done;
			case '!':		flags &= ~QUERY; show = 1;
			}
	}
done:
	if (ret == DSABORT)
		say("Replacement aborted");
	else if (ret == DSBAD) {
		show_replace(str, with, (flags & ~INCR) | BAD);
	} else if ((evershow || (flags & INCR)) && !in_macro())
		say("Replaced %d of %d.", changed, found);
        free (capWith);                 // SER
        free (upWith);                  // SER
	free_spot(orig);
	free_spot(prev);
	return ret;
}
/* End of Epsilon 6.03 */

/* Taken from Epsilon 7.0 sources */

string_replace(str, with, flags)
char *str, *with;
{
	spot orig = alloc_spot(), prev = alloc_spot(), oldmark;
	int skip = 0, show = 0, ret, evershow = 0, oldstart;
        int capitalize = case_fold;     // SER
        char *capWith = malloc(strlen(with)+1); // SER
        char *upWith = malloc(strlen(with)+1); // SER

        if (case_fold) {                // SER
             char *s;                   // SER
             for (s = with; *s && !isupper(*s); *s++) ; // SER
             if (isupper(*s)) capitalize = FALSE; // SER
             }                          // SER

        if (capitalize) {               // SER
             strcpy (capWith, with);    // SER
             strcpy (upWith,  with);    // SER
             capitalize_string (capWith); // SER
             uppercase_string (upWith); // SER
             }

	replace_num_found = replace_num_changed = 0;
	for (;;) {
		skip = 0;
		*prev = point;
		oldstart = matchstart;
		ret = do_searching(flags, str);
		if (!ret || ret == DSBAD) {
			point = *orig;
			break;
		} else if (ret == DSABORT)
			break;
		replace_num_found++;
ask:		if (show) {
			show_replace(str, with, (flags & ~INCR));
			evershow = 1;
			show = 0;
		}
		if (!(flags & QUERY)) goto do_replace;
		get_replace_key(flags, str, skip); // show match, then get key
		if (key == '?' || key == 'h'
					|| root_keys[key] == (short) help) {
			help_on_command("query-replace");
			show = 1;
			point = *prev;
			continue;
		} else if (user_abort || key == abort_key) {
			point = *orig;
			ret = DSABORT;
			goto done;
		}
		switch(tolower(key)) {
			case '^': point = *prev; matchstart = oldstart;
			case '\b':
			case 'n': skip = 1;	/* go to the next match */
			case ' ':
			case 'y':		/* do the replace */
			case '!':
			case ',':
			case '.': break;
			case CTRL('R'):
				say("");
				oldmark = alloc_spot();
				*oldmark = mark;	/* save mark */
				mark = matchstart;
					/* set temporary mark before text */
				recursive_edit();
				if (spot_to_buffer(oldmark) < 0) {
					ret = DSABORT;	// user deleted buf
					goto done;
				}
				mark = *oldmark;	/* restore mark */
				free_spot(oldmark);
				show = 1;
				continue;
			default:	if (set_flag_modes(&flags)) {
						show = 1;
						goto ask;
					}
					if (!IS_ENTER_KEY(key))
						ungot_key = key;
			case ESC:	ret = 1; goto done;
		}
do_replace:	if (!skip) {
			save_spot matchstart;
			if (flags & REGEX)
				fromorig(with);
			else {
                             if (!capitalize) { // SER
                                  delete(matchstart, point);
                                  stuff(with);
                                  }
                             else capital_stuff (strlen(str), with, capWith,
                                                 upWith); // SER
			}
			replace_num_changed++;
			restore_vars();
		}
		if (flags & QUERY)
			switch (key) {
			case ',': case '^':	skip = 1; goto ask;
			case '.':		ret = 1; goto done;
			case '!':		flags &= ~QUERY; show = 1;
			}
	}
done:
	if (ret == DSABORT)
		say("Replacement aborted");
	else if (ret == DSBAD)
		show_replace(str, with, (flags & ~INCR) | BAD);
	else if (!in_macro())
		if (evershow || (flags & INCR))
			say("Replaced %d of %d.", replace_num_changed,
			    replace_num_found);
		else
			say("");
        free (capWith);                 // SER
        free (upWith);                  // SER
	free_spot(orig);
	free_spot(prev);
	return ret;
}
/* End of Epsilon 7.0 */

/* capital_stuff (strlen, char *with, char *capWith, char *upWith)
 * 
 * Replace the 'strlen' characters preceding the point with 'with', doing
 * capitalization if the buffer characters are capitalized.
 *              str     in the buffer with   with [cap unchanged]
 *              Str     in the buffer with   With
 *              STR     in the buffer with   WITH
 * 
 * The decision is made on the first two alphabetic characters of 'str' within
 * the buffer.  The logic of this routine is rather gross, but I can't figure
 * out any efficient way to make it nice.  [Maybe with a Regex?]
 * 
 */
capital_stuff(strlen, with, capWith, upWith)
int strlen;
char *with, *capWith, *upWith;
{
     int strptr = point - strlen;
     char strchar;
     int capital = 0;                   /* Cap type: 0=none 1=Foo 2=FOO */
     
     while (strptr < point && !isalpha(strchar = character(strptr++))) ;
     if (strptr < point) {
          capital += isupper(strchar);
          while (strptr < point && !isalpha(strchar = character(strptr++))) ;
          if (strptr < point) {
               capital += isupper(strchar);
               }
          }
     delete (point - strlen, point);
     switch (capital) {
          case 0: {
               stuff(with);
               break;
               }
          case 1: {
               stuff(capWith);
               break;
               }
          case 2: {
               stuff(upWith);
               break;
               }
          }
}

uppercase_string(s)
char *s;
{
     while (*s) {
          *s = toupper(*s);
          s++;
          }
}

capitalize_string(s)
char *s;
{
	int first = TRUE;
        
        while (*s) {
             if (first) *s = toupper(*s);
             first = !isalpha(*s);
             s++;
             }
}
