
#include "util.h"

/* util.e - by Stephen Robbins
 * 
 * This file contains utility functions used by my Epsilon macros.
 * 2/19/95:  Functions like 'match' can be substantially simplified
 * in later versions of Epsilon.  
 * 
 */

change_chars(s, old, new)
char old, new, *s;
{
     for (; *s != '\0'; s++)
          if (*s == old) *s = new;
}

/* strcat_printable (char *dest, char *source, int len)
   
   If source is longer than 'len,' append 'source' into the destination with
   "..." in the middle of it.  Also, replace all ^Qs with spaces.
   
 */

strcat_printable(dest, source, len)
char *dest, *source;
int len;
{
     int keep, src_len;

     src_len = strlen(source);
     if (src_len > len) {
          keep = (len - 5) / 2;
          strncat(dest, source, keep);
          strcat (dest, " ... ");
          strcat (dest, &source[src_len - keep]);
          }
     else strcat (dest, source);
     change_chars (dest, '\021', ' ');
     change_chars (dest, '\n', ' ');
}

/* int match (char *s, int case_insensitive)
 * Pass:
 *      case_insensitive        non-zero to do a case-insensitive compare
 *
 * Returns:     -1 iff the point is right before the string 's' in the buffer.
 *
 * Implementation note:  I am operating under the assumption that
 * the looping through with the character() function is faster than
 * allocating memory, GRABbing buffer text, STRCMPing, etc.  That
 * asssumption may be wrong.
 */
int match (s, case_insensitive)
char *s;
int case_insensitive;
{
     int slen = strlen(s);
     int idx;

     /* if too close to end of buffer, can't match. */
     if (size() - point < slen) return FALSE;

     if (case_insensitive) {
          for (idx = 0; idx < slen; idx++)
               if (toupper(character(point + idx)) != toupper(s[idx]))
                    return FALSE;
               }
     else { /* no case folding */
          for (idx = 0; idx < slen; idx++)
               if (character(point + idx) != s[idx]) return FALSE;
          }

     return TRUE;
}
